#!/usr/bin/env wish
# vim:set et ai ts=2 syn=tcl:

package require Tcl 8.4
package require Tk
package require Img

set ::factor 1.0
set ::version {uiv}
set ::longver {@VERSION@}

set ::files {}
array set ::options {
  maxdepth    inf
  extensions  {.bmp .gif .jpg .jpeg .png .tif .tbg .xbm .xpm}
  debug       off
  scrollbar   yes
}
set ::options(runcom) "$env(HOME)/.uivrc"

proc log text {if $::options(debug) {puts stderr $text}}
proc enable  {args} {foreach k $args {set ::options($k) 1}}
proc disable {args} {foreach k $args {set ::options($k) 0}}

# array of lambdas to invoke on option parsing
# just expose -v, let the user customize
# every element becomes the rest of the command line passed
set ::optparse(-v) {args {puts "$::version\n$::longver"; exit}}

proc in {list e} {expr {[lsearch -exact $list $e] >= 0}}
proc walk {name {depth 0}} {
  if {! [file exists $name]} {
    return; # ignore imaginary files
  } elseif {$::options(maxdepth) != "inf" && $::options(maxdepth) <= $depth } { ;# recursion?
    return
  }
  # run through a directory and return files with one of $::options(extensions) as extension.
  if {[file isfile $name] && [in $::options(extensions) [string tolower [file extension $name]]]} {
    lappend ::files $name
  } else {
    incr depth
    foreach dirent [lsort [glob -directory $name -nocomplain *]] {
      ::walk $dirent $depth
    }
  }
}

proc openImg {w fn} {
   catch {image delete $::im1}
   set ::im1 [image create photo -file $fn]
   scale $w
   $w xview moveto 0; # ensure the canvas is in it's upper left corner when we load a new image
   $w yview moveto 0; # otherwise the position which was scrolled to before is kept
   join [list [file tail $fn] - [file size $fn] bytes, [image width $::im1]x[image height $::im1]]
}

proc scale {w {n 1}} {
       global im1 im2 factor
       set factor [expr {$factor*$n}]
       $w delete img
       catch {image delete $im2}
       set im2 [image create photo]
       if {$factor>=1} {
          set f [expr int($factor)]
          $im2 copy $im1 -zoom $f $f
       } else {
          set f [expr round(1./$factor)]
          $im2 copy $im1 -subsample $f $f
       }
       $w create image 1 1 -image $im2 -anchor nw -tag img
       $w config -scrollregion [$w bbox all]
}

proc display {which} {
  global files current
  switch $which {
    first { set current 0 }
    last  { set current [expr [llength $files] - 1] }
    next  { if {[incr current] >= [llength $files]} {set current 0} }
    prev  { if {[incr current -1] < 0} {set current [expr [llength $files] - 1]} }
    current { }
    default {
      # go n pictures forward
      if {$which < 0} {set sign -1} else {set sign 1}
      log $current
      incr current [expr $sign * ($which % $current)]
      log $current
    }
  }
  log "current = $current; image = [lindex $files $current]";
  if {$files != {}} {
    wm title . "\[[expr $current + 1]/[llength $files]\] [openImg .c [lindex $files $current]] - $::version"
  }
}

canvas .c -xscrollcommand ".x set" -yscrollcommand ".y set"
scrollbar .x -ori hori -command ".c xview"
scrollbar .y -ori vert -command ".c yview"

# some default keybindings, first for the main window
set keys {
  <space> {display next}
  <h>     {.c xview scroll -1 units}
  <j>     {.c yview scroll +1 units}
  <k>     {.c yview scroll -1 units}
  <l>     {.c xview scroll +1 units}
  <Right> {display next}
  <Left>  {display prev}
  <End>   {display last}
  <Home>  {display first}
  <q>     exit
  <minus> {scale .c 0.5}
  <plus>  {scale .c 2.0}
  <slash> {display current; # reload image }
  <Next>  {display +20; # go 20 images forward }
  <Prior> {display -20; # go 20 images backward }
}

# now for the canvas. line 1 and 2 enable "dragging" the image with the mouse to see other parts of it
# line 3 and 4 enable scrolling with the mousewheel
set canvaskeys {
  <ButtonPress-1> {%W scan mark %x %y}
  <B1-Motion>     {%W scan dragto %x %y 1}

  <Button-4>      {.c yview scroll -1 units}
  <Button-5>      {.c yview scroll +1 units}
}

# source the user configuration file, if available
if [file isfile $::options(runcom)] { 
  source $::options(runcom)
} else {
  puts stderr "WARNING: Configuration file for uiv not found. It is highly recommended that you create one"
}

pack .c -expand 1 -fill both

if $options(scrollbar) {
  pack .y -side right -anchor ne -fill y -expand 1 -in .c
  pack .x -side bottom -fill both -anchor s -in .
}

foreach {key action} $keys       { bind .  $key $action }
foreach {key action} $canvaskeys { bind .c $key $action }

# standard "option parser", can be overridden from the rc-file
# TODO: just pass the arguments which come *after* the option name to the lambda, not all of them
# TODO: make -v work.
foreach elem $argv {
  if [in [array names ::optparse] $elem] {
    apply $::optparse($elem) $argv
  }
  walk $elem
}

log $files
display first
